@@ -112,43 +112,53 @@ UV
 ord_in_utf8(U8 *s, STRLEN curlen, STRLEN *retlen)
 {
     UV uv = 0;
-    int len, i;
+    STRLEN len, i;
+
+    if (curlen == 0) {
+	if (retlen)
+	    *retlen = 0;
+	return 0;
+    }
+
+    len = *s < 0x80 ? 1 :
+	  *s < 0xC0 ? 0 :
+	  *s < 0xE0 ? 2 :
+	  *s < 0xF0 ? 3 :
+	  *s < 0xF8 ? 4 : 0;
+
+    if (curlen < len || len == 0) {
+	if (retlen)
+	    *retlen = 0;
+	return 0;
+    }
 
     if (*s < 0x80) {
 	uv = (UV)*s;
-	len = 1;
-    }
-    else if (*s < 0xC0) {
-	len = 0;
     }
     else if (*s < 0xE0) {
 	uv = (UV)(((s[0] & 0x1f) << 6) | (s[1] & 0x3f));
-	len = 2;
     }
     else if (*s < 0xF0) {
 	uv = (UV)(((s[0] & 0x0f) << 12) |
 		  ((s[1] & 0x3f) <<  6) | (s[2] & 0x3f));
-	len = 3;
     }
     else if (*s < 0xF8) {
 	uv = (UV)(((s[0] & 0x07) << 18) | ((s[1] & 0x3f) << 12) |
 		  ((s[2] & 0x3f) <<  6) |  (s[3] & 0x3f));
-	len = 4;
     }
-    else
-	len = 0;
 
-    for (i = 1; i < len; i++)
+    for (i = 1; i < len; i++) {
 	if (!UTF8A_TRAIL(s[i])) {
 	    len = 0;
 	    break;
 	}
+    }
 
-    if (len != UTF8A_SKIP(uv))
+    if (len != (STRLEN) UTF8A_SKIP(uv))
 	len = 0;
 
     if (retlen)
-	*retlen = (STRLEN)len;
+	*retlen = len;
     return uv;
 }
 
@@ -159,7 +169,6 @@ app_in_utf16le(U8* s, UV uv)
     if (uv <= 0xFFFF) {
 	*s++ = (U8)(uv & 0xff);
 	*s++ = (U8)(uv >> 8);
-	return s;
     }
     else if (Is_VALID_UTF(uv)) {
 	int hi, lo;
@@ -170,10 +179,8 @@ app_in_utf16le(U8* s, UV uv)
 	*s++ = (U8)(hi >> 8);
 	*s++ = (U8)(lo & 0xff);
 	*s++ = (U8)(lo >> 8);
-	return s;
     }
-    else
-	return s;
+    return s;
 }
 
 
@@ -183,7 +190,6 @@ app_in_utf16be(U8* s, UV uv)
     if (uv <= 0xFFFF) {
 	*s++ = (U8)(uv >> 8);
 	*s++ = (U8)(uv & 0xff);
-	return s;
     }
     else if (Is_VALID_UTF(uv)) {
 	int hi, lo;
@@ -194,10 +200,8 @@ app_in_utf16be(U8* s, UV uv)
 	*s++ = (U8)(hi & 0xff);
 	*s++ = (U8)(lo >> 8);
 	*s++ = (U8)(lo & 0xff);
-	return s;
     }
-    else
-	return s;
+    return s;
 }
 
 
@@ -209,10 +213,8 @@ app_in_utf32le(U8* s, UV uv)
 	*s++ = (U8)((uv >>  8) & 0xff);
 	*s++ = (U8)((uv >> 16) & 0xff);
 	*s++ = (U8)((uv >> 24) & 0xff);
-	return s;
     }
-    else
-	return s;
+    return s;
 }
 
 
@@ -224,10 +226,8 @@ app_in_utf32be(U8* s, UV uv)
 	*s++ = (U8)((uv >> 16) & 0xff);
 	*s++ = (U8)((uv >>  8) & 0xff);
 	*s++ = (U8)((uv      ) & 0xff);
-	return s;
     }
-    else
-	return s;
+    return s;
 }
 
 
@@ -236,25 +236,21 @@ app_in_utf8(U8* s, UV uv)
 {
     if (uv < 0x80) {
 	*s++ = (U8)(uv & 0xff);
-	return s;
     }
-    if (uv < 0x800) {
+    else if (uv < 0x800) {
 	*s++ = (U8)(( uv >>  6)         | 0xc0);
 	*s++ = (U8)(( uv        & 0x3f) | 0x80);
-	return s;
     }
-    if (uv < 0x10000) {
+    else if (uv < 0x10000) {
 	*s++ = (U8)(( uv >> 12)         | 0xe0);
 	*s++ = (U8)(((uv >>  6) & 0x3f) | 0x80);
 	*s++ = (U8)(( uv        & 0x3f) | 0x80);
-	return s;
     }
-    if (Is_VALID_UTF(uv)) {
+    else if (Is_VALID_UTF(uv)) {
 	*s++ = (U8)(( uv >> 18)         | 0xf0);
 	*s++ = (U8)(((uv >> 12) & 0x3f) | 0x80);
 	*s++ = (U8)(((uv >>  6) & 0x3f) | 0x80);
 	*s++ = (U8)(( uv        & 0x3f) | 0x80);
-	return s;
     }
     return s;
 }
